/**
 * Created on Apr 15, 2008
 *
 * Copyright 2008 Vuze, Inc.  All rights reserved.
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License only.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA 
 */

package com.aelitis.azureus.activities;

import java.util.HashMap;
import java.util.Map;

import org.gudy.azureus2.core3.internat.MessageText;

import com.aelitis.azureus.buddy.VuzeBuddy;
import com.aelitis.azureus.buddy.impl.VuzeBuddyManager;
import com.aelitis.azureus.util.Constants;
import com.aelitis.azureus.util.MapUtils;

/**
 * @author TuxPaper
 * @created Apr 15, 2008
 *
 */
public class VuzeActivitiesEntryBuddyRequest
	extends VuzeActivitiesEntry
{
	private VuzeBuddy buddy;

	public VuzeActivitiesEntryBuddyRequest() {
		super();
	}

	public VuzeActivitiesEntryBuddyRequest(VuzeBuddy buddy, String acceptURL, long attempNumber) {
		this.buddy = buddy;

		String urlAccept = Constants.appendURLSuffix(acceptURL);

		String textID = "v3.activity.buddy-request";
		if (attempNumber > 1) {
			textID += ".multi";
		}
		String text = MessageText.getString(textID, new String[] {
			buddy.getProfileAHREF(VuzeActivitiesConstants.TYPEID_BUDDYREQUEST),
			urlAccept,
			"" + attempNumber
		});
		
		setText(text);
		setTypeID(VuzeActivitiesConstants.TYPEID_BUDDYREQUEST, true);
		setID(buildID(buddy.getCode()));
	}

	public static String buildID(String code) {
		return VuzeActivitiesConstants.TYPEID_BUDDYREQUEST + "-" + code;
	}

	// @see com.aelitis.azureus.activities.VuzeActivitiesEntry#loadFromExternalMap(java.util.Map)
	public void loadFromExternalMap(Map platformEntry) {
		super.loadFromExternalMap(platformEntry);
		loadOtherValuesFromMap(platformEntry);
	}

	// @see com.aelitis.azureus.activities.VuzeActivitiesEntry#loadFromInternalMap(java.util.Map)
	public void loadFromInternalMap(Map map) {
		super.loadFromInternalMap(map);
		loadOtherValuesFromMap(map);
	}

	private void loadOtherValuesFromMap(Map map) {
		Map mapFutureBuddy = (Map) MapUtils.getMapObject(map, "buddy",
				new HashMap(), Map.class);

		buddy = VuzeBuddyManager.getOrCreatePotentialBuddy(mapFutureBuddy);
	}

	// @see com.aelitis.azureus.activities.VuzeActivitiesEntry#toMap()
	public Map toMap() {
		Map map = super.toMap();

		if (buddy != null) {
			map.put("buddy", buddy.toMap());
		}
		return map;
	}

	public VuzeBuddy getBuddy() {
		return buddy;
	}
}
